(*
    FCE
    Copyright (c) 1996 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit Edit;

interface

uses
  Chord,
  Extra,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls;

type
  TfrmEdit = class(TForm)
    lstFret: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    edtName: TEdit;
    btnOk: TBitBtn;
    btnCancel: TBitBtn;
    pboxMain: TPaintBox;
    pnlHint: TPanel;
    pboxFlag: TPaintBox;
    procedure FormActivate(Sender: TObject);
    procedure pboxMainPaint(Sender: TObject);
    procedure pboxMainMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure pboxMainMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure lstFretChange(Sender: TObject);
    procedure pboxFlagPaint(Sender: TObject);
    procedure pboxFlagMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure pboxFlagMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnOkClick(Sender: TObject);
  private
    { Private declarations }
    SX, SY: integer;
    Center: array[ 1..MAX_FRET, 1..6 ] of TPoint;
    Radius: integer;
    function IsMouseOverDot( X, Y: integer; var F, S: integer ): boolean;
    function IsMouseOverFlag( X, Y: integer; var S: integer ): boolean;
  public
    { Public declarations }
    Chord: TChord;
    CrsNote: HCursor;
  end;

var
  frmEdit: TfrmEdit;

implementation

type
  TNoteName = string[10];
const
  NOTE_NAME : array[ 0..11 ] of TNoteName = (
    'A (la)', 'A# (la#)', 'B (si)', 'C (do)', 'C# (do#)', 'D (re)',
    'D# (re#)', 'E (mi)', 'F (fa)', 'F# (fa#)', 'G (sol)', 'G# (sol#)'
  );
  NOTE_BASE : array[ 1..6 ] of integer = (
    7 (*E*), 2 (*B*), 9 (*G*), 5 (*D*), 0 (*A*), 7 (*E*)
  );
  crArrowAndNote = CUR_NOTE;
  crArrowAndCircle = CUR_RING;
  crArrowAndCross = CUR_SKIP;

{$R *.DFM}

procedure TfrmEdit.FormActivate(Sender: TObject);
var
  F, I, J, X, Y: integer;
begin
  (* Set form data *)
  lstFret.ItemIndex := Chord.Fret - 1;
  edtName.Text := Chord.Name;
  (* Compute some useful global variables *)
  SX := pboxMain.Width div 7;
  SY := (pboxMain.Height-2) div MAX_FRET;
  Radius := (SY*70) div 200;
  Y := 2 + SY div 2;
  for I:=1 to MAX_FRET do begin
    X := 0;
    for J:=6 downto 1 do begin
      Inc( X, SX );
      Center[I,J] := Point( X, Y );
    end;
    Inc( Y, SY );
  end;
  (* Set focus *)
  edtName.SetFocus;
end;

procedure TfrmEdit.pboxMainPaint(Sender: TObject);
var
  I, J, X, Y: integer;
begin
  with pboxMain do begin
    (* Draw the grid *)
    Canvas.Pen.Width := 2;
    for I:=1 to 6 do begin
      Canvas.MoveTo( SX*I, 0 );
      Canvas.LineTo( SX*I, SY*6 );
    end;
    for I:=0 to 6 do begin
      Canvas.MoveTo( SX*1, 1+I*SY );
      Canvas.LineTo( SX*6, 1+I*SY );
    end;
    (* Draw the dots *)
    Canvas.Pen.Width := 1;
    Canvas.Brush.Color := clWhite;
    for I:=1 to MAX_FRET do begin
      for J:=6 downto 1 do begin
        if( (Chord.Data[I] and Mask[J]) <> 0 ) then begin
          X := Center[I,J].X;
          Y := Center[I,J].Y;
          Canvas.Ellipse( X-Radius, Y-Radius, X+Radius, Y+Radius );
        end;
      end;
    end;
  end;
end;

function TfrmEdit.IsMouseOverDot( X, Y: integer; var F, S: integer ): boolean;
var
  I, J, R: integer;
  D, K: longint;
begin
  R := Radius*Radius;
  for I:=1 to MAX_FRET do begin
    for J:=1 to 6 do begin
      K := X - Center[I,J].X;
      D := K*K;
      K := Y - Center[I,J].Y;
      Inc( D, K*K );
      if( D < R ) then begin
        F := I;
        S := J;
        IsMouseOverDot := TRUE;
        Exit;
      end;
    end;
  end;
  IsMouseOverDot := FALSE;
end;

function TfrmEdit.IsMouseOverFlag( X, Y: integer; var S: integer ): boolean;
var
  J, R: integer;
begin
  R := SX div 3;
  for J:=1 to 6 do begin
    if( Abs(X-Center[1,J].X) < R ) then begin
      S := J;
      IsMouseOverFlag := TRUE;
      Exit;
    end;
  end;
  IsMouseOverFlag := FALSE;
end;

procedure TfrmEdit.pboxMainMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var
  F, N, S: integer;
  Hint: string;
begin
  if( IsMouseOverDot( X, Y, F, S ) ) then begin
    (Sender as TPaintBox).Cursor := crArrowAndNote;
    Inc( F, Chord.Fret-1 );
    N := (NOTE_BASE[S] + F) mod 12;
    Hint := ' ' + NOTE_NAME[ N ] + ' at ' + IntToStr( F );
    case F of
      1: Hint := Hint + 'st';
      2: Hint := Hint + 'nd';
      3: Hint := Hint + 'rd';
      else Hint := Hint + 'th';
    end;
    Hint := Hint + ' fret, click to toggle';
    pnlHint.Caption := Hint;
  end
  else begin
    (Sender as TPaintBox).Cursor := crDefault;
    pnlHint.Caption := '';
  end;
end;

procedure TfrmEdit.pboxMainMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  F, S: integer;
begin
  if( IsMouseOverDot( X, Y, F, S ) ) then begin
    Chord.Data[F] := Chord.Data[F] xor MASK[S];
    (Sender as TPaintBox).Refresh;
  end;
end;

procedure TfrmEdit.lstFretChange(Sender: TObject);
begin
  Chord.Fret := lstFret.ItemIndex + 1;
end;

procedure TfrmEdit.pboxFlagPaint(Sender: TObject);
var
  I, J, X, Y: integer;
  RX, RO: integer;
begin
  with pboxFlag do begin
    (* Draw the grid *)
    Canvas.Pen.Width := 2;
    Canvas.Pen.Color := clBlue;
    for I:=2 to 5 do begin
      Canvas.MoveTo( SX*I, 0 );
      Canvas.LineTo( SX*I, Height );
    end;
    Canvas.Brush.Style := bsClear;
    Canvas.Rectangle( SX*1, 1, SX*6+1, Height );
    Canvas.Pen.Color := clBlack;
    (* Draw the data *)
    RX := (Height * 25) div 100;
    RO := (Height * 30) div 100;
    Y := Height div 2 + 1;
    for I:=1 to 6 do begin
      X := Center[1,I].X;
      if( (Chord.Ring and MASK[I]) <> 0 ) then begin
        Canvas.Pen.Width := 3;
        Canvas.Ellipse( X-RO, Y-RO, X+RO, Y+RO );
      end
      else if( (Chord.Skip and MASK[I]) <> 0 ) then begin
        Canvas.Pen.Width := 2;
        Canvas.MoveTo( X-RX, Y-RX );
        Canvas.LineTo( X+RX, Y+RX );
        Canvas.MoveTo( X+RX, Y-RX );
        Canvas.LineTo( X-RX, Y+RX );
      end;
    end;
  end;
end;

procedure TfrmEdit.pboxFlagMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var
  S: integer;
begin
  if( IsMouseOverFlag( X, Y, S ) ) then begin
    if( (Chord.Ring and MASK[S]) <> 0 ) then
      (Sender as TPaintBox).Cursor := crArrowAndCross
    else if( (Chord.Skip and MASK[S]) <> 0 ) then
      (Sender as TPaintBox).Cursor := crArrowAndNote
    else
      (Sender as TPaintBox).Cursor := crArrowAndCircle;
  end
  else begin
    (Sender as TPaintBox).Cursor := crDefault;
  end;
end;

procedure TfrmEdit.pboxFlagMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  S: integer;
begin
  if( IsMouseOverFlag( X, Y, S ) ) then begin
    S := MASK[S];
    if( (Chord.Ring and S) <> 0 ) then begin
      Chord.Ring := Chord.Ring xor S;
      Chord.Skip := Chord.Skip or S;
    end
    else if( (Chord.Skip and S) <> 0 ) then begin
      Chord.Skip := Chord.Skip xor S;
    end
    else begin
      Chord.Ring := Chord.Ring or S;
    end;
    pboxFlag.Refresh;
    pboxFlagMouseMove( Sender, Shift, X, Y );
  end;
end;

procedure TfrmEdit.btnOkClick(Sender: TObject);
begin
  Chord.Name := edtName.Text;
end;

end.
